// Scroll Traces
// $Author: Jeffrey J Weimer$

#pragma rtGlobals=3
#pragma IgorVersion=8.0
#pragma version=2.00

#pragma IndependentModule=ScrollTraces

// for project updater
Static Constant kProjectID=1110
Static StrConstant ksShortTitle="ScrollTraces"

StrConstant thePackage="ScrollTraces"
StrConstant thePackageFolder="root:Packages:ScrollTraces"
StrConstant theProcedureFile = "ScrollTraces.ipf"
StrConstant thePackageInfo = "A control panel to scroll through multiple traces in a graph window"
StrConstant thePackageAuthor = "Jeffrey J Weimer"
Constant thePackageVersion = 2.00
//Constant hasHelp = 0
//Constant removable = 1

// Panel Size Parameters

Constant ndepth = 145
Constant nwidth = 270
Constant cdepth = 50
Constant cwidth = 270
Constant lwidth = 270
StrConstant pprefix = "STP_"

// menu

Menu "Graph", hideable
	Submenu "Panels"
		"Scroll Traces", /Q, SetupScrollTracesPanel()
	end
end

// Preferred Access Point from External Calls
// ScrollTraces#ShowPanel([activate,collapse])

Function ShowPanel([activate,collapse])
	variable activate, collapse

	STRUCT WMCheckboxAction cba
	STRUCT WMButtonAction ba
	
	activate = ParamIsDefault(activate) ? 0 : 1
	collapse = ParamIsDefault(collapse) ? 0 : 1
	
	SetupScrollTracesPanel()
	if (activate)
		cba.eventCode=2
		TurnOnTraceScrollMode(cba)
	endif
	
	if (collapse)
		ba.eventCode=2
		CloseScrollTracePanel(ba)	
	endif
	
	return 0
end

// setup the scroll trace package

Function SetupScrollTracesPanel()

	// create data folder

	DFREF cdf = GetDataFolderDFR()
	if (!DataFolderExists(thePackageFolder))
		SetDataFolder root:
		NewDataFolder/O/S Packages
		NewDataFolder/S $thePackageFolder
		variable/G moviemode=0
		string/G currWin="", plugIns=""
		SetDataFolder cdf
	endif		
	
	// create or show panel
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	SVAR/SDFR=$thePackageFolder cw = currWin
	
	// cannot attach a new panel while in movie mode
	
	if (mm)
		DoWindow/F $cw
		return 0
	endif
	
	// panel attached according to front window name
	
	string fWin = WinName(0,1,1), dfWin
	dfWin = thePackageFolder + ":" + fWin
	
	if (!DataFolderExists(dfWin))
		MakeScrollTracesPanel(fWin)
	endif
	
	return 0
	
end

// make the panel

Function MakeScrollTracesPanel(fWin)
	string fWin
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	SVAR/SDFR=$thePackageFolder cw = currWin
	
	cw = fWin
	mm = 0
		
	// create globals for new panel
	
	DFREF cdf = GetDataFolderDFR()
	
	SetDataFolder $thePackageFolder	
	NewDataFolder/S $cw
	variable/G active, tracenumber, tracehold, traceposition, tracewidth
	variable/G speed, loopmode, scalemode, limitrange, externalmode, keyboard
	variable/G xmin, xmax
	string/G panelName
	make/n=1 scrt_frozen=NaN
	
	NVAR ac = active
	NVAR tn = tracenumber
	NVAR th = tracehold
	NVAR tp = traceposition
	NVAR tw = tracewidth
	NVAR sp = speed
	NVAR lm = loopmode
	NVAR sm = scalemode
	NVAR lr = limitrange
	NVAR xm = externalmode
	NVAR kb = keyboard
	NVAR xmin, xmax
	SVAR pn = panelName
	string vers
	variable ntraces
	
	ac = 0
	tn = 0
	th = inf
	tp = 0
	tw = 0
	sp = 1
	lm = 0
	sm = 1
	lr = 0
	xm = 0
	kb = 1
	sprintf pn, "%s%s", pprefix, cw
	
	if (strlen(pn)>30)
		pn = pn[0,29]
	endif
	
	ntraces = ItemsInList(scrtTraceNameList())
	
	// return to current data folder
	
	SetDataFolder cdf
	
	// set up and attach panel to current graph
	
	DoWindow/F $cw

	SetAxis/W=$cw/A bottom
	DoUpdate/W=$cw
	GetAxis/W=$cw/Q bottom
	xmax = v_max
	xmin = v_min
	
	NewPanel/W=(0,0,nwidth,ndepth)/N=$pn/K=2/EXT=2/HOST=# as "Scroll Traces"
	ModifyPanel noEdit=1, fixedsize=1
	
	Slider slidetraceposition pos={20,10},size={nwidth-120,5}, vert=0,side=0, limits={0,1,0.01}, proc=SlideThruTraces, variable=tp
	Slider slidetraceposition, help={"Slide along the x-axis for traces on the graph."}

	Slider slidetracewidth pos={nwidth-95,10},size={45,5}, vert=0,side=0, limits={-2,0,0.1}, proc=SlideThruTraces, variable=tw
	Slider slidetracewidth, help={"Set the viewport x-axis width."}

	CheckBox scale_r,pos={nwidth-50,7},size={50,14},disable=2,value=lr,proc=SetScaleMode,title=">|\\f01r\\f00|<",font="Arial",value=0
	CheckBox scale_r,help={"Hold the y-axis scale at the current manually chosen range."}
	
	CheckBox onoff,pos={5,15},size={16,34},proc=TurnOnTraceScrollMode,title="",value= 0
	CheckBox onoff, help={"Turn on / off the scroll traces actions."}
	
	Slider scrolltracestack pos={20,30}, size={nwidth-60,5}, vert=0,side=0, limits={0,1,1}, proc=SlideThruTraces, variable=tn
	Slider scrolltracestack, help={"Slide through the different traces on the graph."}

	Button XtendD,pos={4,35},size={15,20},proc=XtendDown,title="v", userData="opened"
	Button XtendD, help={"Open or collapse the panel controls."}
	
	if (cmpstr(IgorInfo(2),"Macintosh")==0)
		Button ClosePanel, pos={nwidth-20,25}, size={15,15}, fsize=8, title="x", proc=CloseScrollTracePanel
		Button ClosePanel, help={"Close the panel."}
	endif

	SetDrawLayer ProgBack
	DrawLine 22,45,nwidth-20,45	
	DrawLine 184,50,184,ndepth-10
	SetDrawEnv fsize=8;
	sprintf vers "v %01.2f", thePackageVersion
	DrawText 5,ndepth-5,vers
	sprintf vers "/ %d", ntraces - 1
	SetDrawEnv fsize=10;
	DrawText 110,70,vers
	SetDrawLayer UserFront
	
	Button first,pos={14,55},size={20,20},disable=2,proc=StepToTrace,title="|<",font="Arial"
	Button first, help={"Go to first trace in set."}
	Button previous,pos={39,55},size={20,20},disable=2,proc=StepToTrace,title="<",font="Arial"
	Button previous, help={"Go to previous trace in set."}

	SetVariable gototrace,pos={65,56},size={40,18},disable=2,proc=GoToTraceNumber,title=" "
	SetVariable gototrace,font="Arial",limits={0,89,0},value=tn

	Button next,pos={130,55},size={20,20},disable=2,proc=StepToTrace,title=">",font="Arial"
	Button next, help={"Go to next trace in set."}
	Button last,pos={155,55},size={20,20},disable=2,proc=StepToTrace,title=">|",font="Arial"
	Button last, help={"Go to last trace in set."}

	Slider speed,pos={61,98},size={75,6},disable=2, variable=sp
	Slider speed,limits={1,20,1},value=1,side= 0,vert= 0
	Slider speed, help={"Set the movie play speed."}
	
	CheckBox loop,pos={102,79},size={42,14},disable=2,proc=SetLoopMode,title="\\f01l\\f00oop",font="Arial"
	CheckBox loop,value=0,mode=1

	CheckBox once,pos={50,79},size={44,14},disable=2,proc=SetLoopMode,title="\\f01o\\f00nce",font="Arial"
	CheckBox once,value= 1,mode=1

	Button LoopForward,pos={152,80},size={25,20},disable=2,proc=DoLoop,title=">>",font="Arial"
	Button LoopForward, help={"Start movie mode to loop forward in the sets of traces."}
	Button LoopBack,pos={18,80},size={25,20},disable=2,proc=DoLoop,title="<<",font="Arial"
	Button LoopBack, help={"Start movie mode to loop backward in the sets of traces."}

	CheckBox scale_a,pos={190,50},size={29,14},value=1,proc=SetScaleMode,title="\\f01a\\f00ll",font="Arial"
	CheckBox scale_a,value=1,mode=1,help={"Auto-scale y-axis to maximum limits across all traces."}
	CheckBox scale_c,pos={190,65},size={52,14},disable=2,value=0,proc=SetScaleMode,title="\\f01c\\f00urrent",font="Arial"
	CheckBox scale_c,value=0,mode=1,help={"Auto-scale y-axis to current trace."}
	CheckBox scale_h,pos={190,85},size={50,14},disable=2,value=0,proc=SetScaleMode,title="\\f01h\\f00old",font="Arial"
	CheckBox scale_h,value=0,mode=1,help={"Hold the y-axis scale at the range for the current trace."}
	CheckBox scale_f,pos={190,100},size={50,14},disable=2,value=0,proc=SetScaleMode,title="\\f01f\\f00reeze",font="Arial"
	CheckBox scale_f,value=0,mode=1,help={"Hold the y-axis scale at the range for the current trace and show current trace."}
	
	SetVariable fhtrace, pos={nwidth-30,90},size={25,15}, font="Arial", fsize=8, limits={0,89,0}, value=th, disable=2, title=" "
	SetVariable fhtrace, help={"Trace that is being used to hold scale or show as a frozen trace."}
	
	CheckBox keyboard, pos={16,110}, title="@", font="Arial", fsize=10, disable=2, value=1, proc=ToggleKeyboard
	CheckBox keyboard, help={"Use keyboard inputs to control actions."}
	
	PopupMenu external, pos={55,115}, mode=1, title="", font="Arial", value=get_NPlugInsMenu(), fsize=0, disable=2, proc=ChangeExternalMode
	PopupMenu external, help={"Plug in that will be run each time a new trace is shown."}
	
	Button GetHelp, pos={155,115}, size={20,20}, title="?", font="Arial", fsize=10, proc=GetHelp
	Button GetHelp, help={"Get help."}
	
	// activate the basic window hooks
	
	SetWindowHooks(0)

	return 0
end

// slide through traces

Function SlideThruTraces(sa) : SliderControl
	STRUCT WMSliderAction &sa

	switch( sa.eventCode )
		case -1: // kill
			break
		default:
			strswitch(sa.ctrlname)
				case "scrolltracestack":
					if( sa.eventCode & 1 ) // value set
						DoAtTheTrace(sa.curval)
					endif
					break
				case "slidetraceposition":
				case "slidetracewidth":
					ChangeViewPort()
					break	
			endswitch				
			break
	endswitch

	return 0
End

Function ChangeViewPort()

	SVAR cw = $(thePackageFolder + ":currWin")
	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR xn = cwDFR:xmin
	NVAR xx = cwDFR:xmax
	NVAR lr = cwDFR:limitrange
	NVAR tw = cwDFR:tracewidth
	NVAR tp = cwDFR:traceposition
	SVAR pn =  cwDFR:panelName
	
	variable aleft, aright, awidth, vwidth
	string cpanel = cw + "#" + pn

	if (tw == 0)
		tp = 0
		lr = 0
		Checkbox scale_r win=$cpanel, disable=2, value=lr
		SetAxis/W=$cw/A left
		return 0
	endif
	
	awidth = xx - xn
	vwidth = (10^tw)*awidth
	aleft = tp*(awidth - vwidth)
	aright = aleft + vwidth
	SetAxis/W=$cw bottom, aleft, aright
	Checkbox scale_r win=$cpanel, disable=0
	
	return 0
end

// turn on/off the panel operation

Function TurnOnTraceScrollMode(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	
	// cannot activate a panel when another one is in movie mode
	
	if (mm)
		return 0
	endif

	SVAR cw = $(thePackageFolder + ":currWin")
	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR ac = cwDFR:active
	NVAR tn = cwDFR:tracenumber
	NVAR th = cwDFR:tracehold
	NVAR tp = cwDFR:traceposition
	NVAR tw = cwDFR:tracewidth
	NVAR sp = cwDFR:speed
	NVAR lm = cwDFR:loopmode
	NVAR sm = cwDFR:scalemode
	NVAR lr = cwDFR:limitrange
	NVAR xm = cwDFR:externalmode
	NVAR kb = cwDFR:keyboard
	SVAR pn = cwDFR:panelName
	
	string cpanel = cw + "#" + pn

	switch( cba.eventCode )
		case 2: // mouse up
			variable nt = ItemsInList(scrtTraceNameList())
			ac = cba.checked
			tn = 0
			th = inf
			//tp = 0
			//tw = 0
			sp = 1
			lm = 0
			sm = 1
			xm = 0
			kb = 1
			if (ac)
				// activating
				if (nt==0)
					CheckBox $cba.ctrlName win=$cpanel, value = 0
					break
				endif
				nt += -1
				string theOne = StringFromList(0,scrtTraceNameList())
				SetVariable gotoTrace win=$cpanel, disable=0,limits={0,nt,0}
				Slider scrolltracestack win=$cpanel, disable=0, limits={0,nt,1}
				Button first win=$cpanel, disable=0
				Button previous win=$cpanel, disable=0
				Button next win=$cpanel, disable=0
				Button last win=$cpanel, disable=0
				Button LoopForward win=$cpanel, disable=0
				Button LoopBack win=$cpanel, disable=0
				Slider speed win=$cpanel, disable=0, value=sp
				CheckBox loop win=$cpanel, disable=0, value=0
				CheckBox once win=$cpanel, disable=0, value=1
				Checkbox scale_a win=$cpanel, disable=0, value=1
				Checkbox scale_c win=$cpanel, disable=0, value=0
				Checkbox scale_r win=$cpanel, disable=(2*(tw==0))
				Checkbox keyboard win=$cpanel, disable=0, value=1
				PopupMenu external win=$cpanel, disable=0
				SetVariable fhtrace win=$cpanel, disable=0,limits={0,nt,0}
				// hide all the traces then show the one to use
				ModifyGraph/W=$cw hideTrace=2
				ModifyGraph/W=$cw hideTrace($theOne)=0
			else
				// deactivating
				SetVariable gotoTrace win=$cpanel, disable=2
				Slider scrolltracestack win=$cpanel, disable=2
				Button first win=$cpanel, disable=2
				Button previous win=$cpanel, disable=2
				Button next win=$cpanel, disable=2
				Button last win=$cpanel, disable=2
				Button LoopForward win=$cpanel, disable=2
				Button LoopBack win=$cpanel, disable=2
				Slider speed win=$cpanel, disable=2, value=1
				CheckBox loop win=$cpanel, disable=2, value=0
				CheckBox once win=$cpanel, disable=2, value=1
				Checkbox scale_a win=$cpanel, disable=0, value=1
				Checkbox scale_c win=$cpanel, disable=2, value=0
				Checkbox scale_h win=$cpanel, disable=2, value=0
				Checkbox scale_f win=$cpanel, disable=2, value=0
				Checkbox scale_r win=$cpanel, disable=(2*(tw==0))
				Checkbox keyboard win=$cpanel, disable=2, value=1
				PopupMenu external win=$cpanel, disable=2
				SetVariable fhtrace win=$cpanel, disable=2
				// show all the traces
				ModifyGraph/W=$cw hideTrace=0
				SetAxis/W=$cw/A left
			endif
//			SetActiveSubWindow ##
			break
	endswitch

	return 0
end
// extend the panel down
 
Function XtendDown(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode

	// cannot activate a panel when another one is in movie mode
	
	if (mm)
		return 0
	endif

	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	SVAR pn = cwDFR:panelName
 
	switch( ba.eventCode )
		case 2: // mouse up
			strswitch(ba.userData)
				case "closed":
					Button XtendD win=$(cw + "#" + pn), title="v", userData="opened"
					MoveSubWindow/W=$(cw + "#" + pn) fnum=(0,0,nwidth,ndepth)
					break
				case "opened":
					Button XtendD win=$(cw + "#" + pn), title=">", userData="closed"
					MoveSubWindow/W=$(cw + "#" + pn) fnum=(0,0,cwidth,cdepth)
					break
			endswitch
			break
	endswitch
	return 0
end

// set the graph active to recognize it has an attached panel

Function SetWindowHooks(how)
	variable how
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	SVAR pn = cwDFR:panelName
	string cpanel = cw + "#" + pn
	
	switch(how)
		case -1:	// clear
			SetWindow $cw hook(ScrTHook)=$""
			SetWindow $cw hook(ScrTKeyboard)=$""
			SetWindow $cw hook(ScrTMovieMode)=$""
			SetWindow $cpanel hook(ScrTHook)=$""
			SetWindow $cpanel hook(ScrTKeyboard)=$""
			SetWindow $cpanel hook(ScrTMovieMode)=$""
			break			
		case 0:	// set active non-movie mode
			SetWindow $cw hook(ScrTHook)=ScrollTracesHook
			SetWindow $cw hook(ScrTKeyboard)=ScrollTracesKeyboardHook
			SetWindow $cw hook(ScrTMovieMode)=$""
			SetWindow $cpanel hook(ScrTHook)=ScrollTracesHook
			SetWindow $cpanel hook(ScrTKeyboard)=ScrollTracesKeyboardHook
			SetWindow $cpanel hook(ScrTMovieMode)=$""
			break
		case 1:	// set active movie mode
			SetWindow $cw hook(ScrTHook)=$""
			SetWindow $cw hook(ScrTKeyboard)=$""
			SetWindow $cw hook(ScrTMovieMode)=ScrollTracesMovieModeHook
			SetWindow $cpanel hook(ScrTHook)=$""
			SetWindow $cpanel hook(ScrTKeyboard)=$""
			SetWindow $cpanel hook(ScrTMovieMode)=ScrollTracesMovieModeHook
			break
	endswitch
	
	return 0
end

// Button Action to Get Help

Function GetHelp(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode

	switch( ba.eventCode )
		case 2:
			if (mm)
				return -1
			endif
			get_ProcHelp("ScrollTracesHelp")
			break
	endswitch

	return 0
End

// procedure to get help

Function get_ProcHelp(string helpName)
	
	string fldrPath, helpfile = helpName + ".ihf"
	fldrPath = ParseFilePath(1,FunctionPath(""),":",1,0)
	NewPath/Z/Q/O helpName, fldrPath 
	OpenHelp/P=helpName helpfile
	KillPath/Z helpName
	
	return 0
end

// change external function mode

Function ChangeExternalMode(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa

	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR xm = cwDFR:externalmode

	switch( pa.eventCode )
		case 2: // mouse up
			xm = pa.popNum-1
			LoadScrollTracesPlugIns(xm)
			break
	endswitch

	return 0
End

// load/unload plugins

Function LoadScrollTracesPlugIns(how)
	variable how
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	SVAR/Z pl = $(thePackageFolder + ":plugIns")
	NVAR/Z tn = cwDFR:tracenumber

	string dirpath="", plugInFiles="", theOne="", plugIns="", theCmd=""
	variable ic=0
	
	switch(how)
		case 0:
		case 1:
			if (strlen(pl)==0)
				break
			endif
			plugInFiles = FunctionList("ScrTracesPI_*",";","WIN:[ProcGlobal]")
			if (how==1)
				plugInFiles = RemoveFromList("ScrTracesPI_Default",plugInFiles)
			endif
			ic = 0
			do
				theCmd=StringFromList(ic,plugInFiles)
				if (strlen(theCmd)==0)
					break
				endif
				theOne = StringFromList(tn,scrtTraceNameList())
				sprintf theCmd, "ProcGlobal#%s(1,\"%s\",\"%s\",%d)", theCmd, cw, theOne, tn
				Execute/Q/Z theCmd
				ic += 1
			while(1)
			ic = 0
			do
				theOne = StringFromList(ic,pl)
				if (strlen(theOne)==0)
					break
				endif
				sprintf theCmd, "DELETEINCLUDE \"%s\"", theOne
				Execute/Q/Z/P theCmd
				sprintf theCmd, "CloseProc/NAME=\"%s.ipf\"", theOne
				Execute/Q/Z/P theCmd
				ic +=1
			while(1)
			pl = ""
			break
		case 2:
			theOne = GetIndependentModuleName() + "#MakeScrollTracesPanel"
			theCmd = "setigoroption independentmoduledev=1"
			Execute/Q/Z theCmd
			dirpath = ParseFilePath(1,FunctionPath(theOne),":",1,0) 
			theCmd = "setigoroption independentmoduledev=0"
			Execute/Q/Z theCmd		
			if (strlen(dirpath)==0)
				break
			endif
			dirpath += "ScrTraces PlugIns"
			NewPath/O/Q ScrollTracesPlugIns, dirpath
			plugInFiles = IndexedFile(ScrollTracesPlugIns,-1,"????")
			ic = 0
			do
				theOne = ParseFilePath(3,StringFromList(ic,plugInFiles),";",0,0)
				if (strlen(theOne)==0)
					break
				endif
				// remove another extension in case windows alias
				theOne = ParseFilePath(3,theOne,";",0,0)
				sprintf theCmd, "INSERTINCLUDE \"%s\"", theOne
				Execute/Q/Z/P theCmd
				plugIns += theOne + ";"
				ic +=1
			while(1)
			Execute/Q/P/Z "COMPILEPROCEDURES "
			KillPath/Z ScrollTracesPlugIns
			pl = plugIns
			ic = 0
			do
				theCmd=StringFromList(ic,pl)
				if (strlen(theCmd)==0)
					break
				endif
				theOne = StringFromList(tn,scrtTraceNameList())
				sprintf theCmd, "ProcGlobal#%s(0,\"%s\",\"%s\",%d)", theCmd, cw, theOne, tn
				Execute/Q/Z/P theCmd
				ic += 1
			while(1)
			break
	endswitch
	Execute/Q/Z/P "COMPILEPROCEDURES "
	
	return 0
end

// close the panel (Macintosh only)

Function CloseScrollTracePanel(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode

	switch(ba.eventCode)
		case 2:		// button down
			if (mm)
				break
			endif
			CleanUpandKill()
			break	
	endswitch
	return 0
end

// trace name list excluding special scrt_frozen

Function/S scrtTraceNameList([variable rs])
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	if (ParamIsDefault(rs))
		rs = 1
	endif
	string theList
	theList = GrepList(TraceNameList(cw,";",1),"scrt_frozen*",rs)

	return (theList)
end

// advance thru traces forward or backward via buttons

Function StepToTrace(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	
	if (mm)
		return 0
	endif

	SVAR cw = $(thePackageFolder + ":currWin")
	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR tn = cwDFR:tracenumber
	
	switch( ba.eventCode )
		case 2:
			strswitch(ba.ctrlName)
				case "first":
					tn = 0
					break
				case "next":
					tn += 1
					break
				case "last":
					tn = inf
					break
				case "previous":
					tn += -1
					break					
			endswitch
			STRUCT WMSetVariableAction sva
			sva.eventCode=99
			GoToTraceNumber(sva)
			break
	endswitch
	
	return 0
end

// go to a particular trace number using buttons, setvariable input, or movie input

Function GoToTraceNumber(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR tn = cwDFR:tracenumber
	NVAR lm = cwDFR:loopmode

	variable nt
	
	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
			DoAtTheTrace(sva.dval)
			break
		case 99: // left/right cursor key or movie mode
			nt = ItemsInList(scrtTraceNameList())
			if (tn < 0)
				tn = lm==0 ? 0 : nt-1
			endif
			if (tn >= nt)
				tn = lm==0 ? nt-1 : 0
			endif
			DoAtTheTrace(tn)
			break
	endswitch
	
	return 0
end

Function DoAtTheTrace(theN)
	variable theN
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR/Z tn = cwDFR:tracenumber
	NVAR/Z sm = cwDFR:scalemode
	NVAR/Z xm = cwDFR:externalmode
	
	string theOne = StringFromList(theN,scrtTraceNameList()), theFList="", theCmd=""
	variable howMany
	
	// do actions
	
	SetScaleState()
	tn = theN
	
	// do external plug-in
	
	switch(xm)
		case 0:
			break
		case 1:
			theFList = FunctionList("ScrTracesPI_Default",";","WIN:[ProcGlobal]")
			if (strlen(theFList)==0)
				break
			endif
			sprintf theCmd, "ProcGlobal#ScrTracesPI_Default(%d,\"%s\",\"%s\",%d)", sm, cw, theOne, tn
			Execute/Q/Z theCmd
			break
		case 2:
			theFList = FunctionList("ScrTracesPI_*",";","WIN:[ProcGlobal]")
			howMany = ItemsInList(theFList)
			for (theN=0;theN<howMany;theN+=1)
				theCmd=StringFromList(theN,theFList)
				sprintf theCmd, "ProcGlobal#%s(%d,\"%s\",\"%s\",%d)", sm, theCmd, cw, theOne, tn
				Execute/Q/Z theCmd
			endfor
			break
	endswitch
	
	DoUpdate/W=$cw

	return 0	
end

// turn on/off window hook functions for keyboard input

Function ToggleKeyboard(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR kb = cwDFR:keyboard

	switch( cba.eventCode )
		case 2: // mouse up
			kb = cba.checked
			break
	endswitch
	return 0
end

// select how the ordinate is to be scaled using the panel checkboxes

Function SetScaleMode(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	wave sctf = cwDFR:scrt_frozen
	SVAR pn = cwDFR:panelName
	NVAR sm = cwDFR:scalemode
	NVAR lr = cwDFR:limitrange
	NVAR tn = cwDFR:tracenumber
	NVAR th = cwDFR:tracehold
	NVAR tw = cwDFR:tracewidth
	
	string theOne="", cpanel = cw + "#" + pn
	variable sf
	lr = cba.checked
	
	switch( cba.eventCode )
		case 99: // manual change
			Checkbox $cba.ctrlName win=$cpanel, value=1
		case 2: // mouse up
			sf = lr == 0 ? 1 : -1
			strswitch(cba.ctrlName)
				case "scale_a":
					th = inf
					SetVariable fhtrace win=$cpanel, value=th
					Checkbox scale_c win=$cpanel, value=0
					Checkbox scale_h win=$cpanel, value=0, disable=2
					Checkbox scale_f win=$cpanel, value=0, disable=2
//					if (tw == 0)
//						lr = 0
//						Checkbox scale_r win=$cpanel, disable=2, value=0
//					else
//						Checkbox scale_r  win=$cpanel, disable=0
//					endif
					sm = 1*sf
					break
				case "scale_c":
					th = tn
					SetVariable fhtrace win=$cpanel, value=th
					Checkbox scale_a win=$cpanel, value=0
					Checkbox scale_h win=$cpanel, value=0, disable=0
					Checkbox scale_f win=$cpanel, value=0, disable=0
//					if (tw == 0)
//						Checkbox scale_r win=$cpanel, disable=2, value=0
//					else
//						Checkbox scale_r  win=$cpanel, disable=0
//					endif
					sm = 2*sf
					break
				case "scale_h":
					th = tn
					lr = 0
					SetVariable fhtrace win=$cpanel, value=th
					Checkbox scale_c win=$cpanel, value=0
					Checkbox scale_f  win=$cpanel, value=0
					Checkbox scale_r  win=$cpanel, disable=2
					sm = 3*sf
					break
				case "scale_f":
					lr = 0
					Checkbox scale_c win=$cpanel, value=0
					Checkbox scale_h  win=$cpanel, value=0
					Checkbox scale_r  win=$cpanel, disable=2
					sm = 4*sf
					break
				case "scale_r":
					sm = sf*abs(sm)
					break
			endswitch
			RemoveFromGraph/W=$cw/Z scrt_frozen
			SetScaleState()
			break
	endswitch
	
	return 0
end

// set how the ordinate is to be scaled (while advancing thru the traces)
// the current trace is shown already

Function SetScaleState()

	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	wave sctf = cwDFR:scrt_frozen
	NVAR ac = cwDFR:active
	NVAR sm = cwDFR:scalemode
	NVAR tn = cwDFR:tracenumber
	NVAR th = cwDFR:tracehold
	
	string theOne, theHoldOne
	variable as=1
	
	if (sm<0)
		as=2
	endif

	theOne = StringFromList(tn,scrtTraceNameList())
	
	switch(abs(sm))
		case 1:	// all
			if (ac)
				ModifyGraph/W=$cw hideTrace=2, hideTrace($theOne)=0
			endif
			SetAxis/A=(as) left
			break
		case 2:	// current
			ModifyGraph/W=$cw hideTrace=1, hideTrace($theOne)=0
			SetAxis/A=(as) left
			break
		case 3:	// hold
			ModifyGraph/W=$cw hideTrace=1, hideTrace($theOne)=0
//			SetAxis/A=(as) left
//			DoUpdate/W=$cw
			GetAxis/Q left
			SetAxis left V_min,V_max
			break
		case 4:	// freeze
			if (ItemsInList(scrtTraceNameList(rs=0))==0)
				th = tn
				theHoldOne = StringFromList(th,scrtTraceNameList())
				duplicate/O TraceNametoWaveRef(cw,theHoldOne) sctf
				if (WaveExists(XWaveRefFromTrace(cw,theHoldOne)))
					wave wx = XWaveRefFromTrace(cw,theHoldOne)
					appendtograph sctf vs wx
				else
					appendtograph sctf
				endif
			endif
			ModifyGraph/W=$cw hideTrace=1, hideTrace($theOne)=0
			ModifyGraph/Z/W=$cw hideTrace(scrt_frozen)=0, lstyle(scrt_frozen)=1, rgb(scrt_frozen)=(0,0,0)
//			SetAxis/A=(as) left
//			DoUpdate/W=$cw
			GetAxis/Q left
			SetAxis left V_min,V_max
			break
	endswitch
	
	return 0
end

// set loop once or multiple times

Function SetLoopMode(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR lm = cwDFR:loopmode
	SVAR pn = cwDFR:panelName

	switch( cba.eventCode )
		case 99:
			CheckBox $cba.ctrlName win=$(cw + "#" + pn), value = 1
		case 2: // mouse up
			strswitch(cba.ctrlName)
				case "once":
					CheckBox loop win=$(cw + "#" + pn), value=0
					lm = 0
					break
				case "loop":
					Checkbox once win=$(cw + "#" + pn), value=0
					lm = 1
					break
			endswitch
			break
	endswitch

	return 0
end

// start loop thru traces

Function DoLoop(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR lm = cwDFR:loopmode
	SVAR pn = cwDFR:panelname
	NVAR tn = cwDFR:tracenumber

	variable fw = 1, st
	string cpanel = cw + "#" + pn

	// disable movie mode for now!
	
//	return 0
	
	switch( ba.eventCode )
		case 2:
//			print "DoLoop ", tn, mm
			if (!mm)
				strswitch(ba.ctrlName)
					case "LoopForward":
						Button LoopBack win=$cpanel, disable=2
						break
					case "LoopBack":
						Button LoopForward win=$cpanel, disable=2
						fw = 0
						break
				endswitch
				Button $ba.ctrlName win=$cpanel, title="X", userdata="Active", disable=0, noproc
				st = 2
			else
				mm = 0
				DoUpdate/W=$(cw + "#" + pn)/E=0
				SetWindowHooks(0)
				strswitch(ba.ctrlName)
					case "LoopForward":
						Button LoopBack win=$cpanel, disable=0
						Button LoopForward win=$cpanel, title=">>", userdata="", proc=DoLoop
						break
					case "LoopBack":
						Button LoopForward win=$cpanel, disable=0
						Button LoopBack win=$cpanel, title="<<", userdata="", proc=DoLoop
						break
				endswitch
				ba.UserData = ""
				st = 0
			endif
			CheckBox once win=$cpanel, disable=st
			CheckBox loop win=$cpanel, disable=st
			Button first win=$cpanel, disable = st
			Button next win=$cpanel, disable = st
			Button previous win=$cpanel, disable = st
			Button last win=$cpanel, disable = st
			Checkbox scale_a win=$cpanel, disable=st
			Checkbox scale_f win=$cpanel, disable=st
			Checkbox scale_h win=$cpanel, disable=st
			Checkbox scale_c win=$cpanel, disable=st
			Checkbox onoff win=$cpanel, disable=st
			Checkbox keyboard win=$cpanel, disable=st
			PopupMenu external win=$cpanel, disable=st
			Button GetHelp win=$cpanel, disable=st
			//Button RemoveMe win=$(cw + "#" + pn), disable=st
			if (st == 2)
				mm = 1
				SetWindowHooks(1)
				if (lm==1)
					do
						LoopThruTraces(ba)
						if (cmpStr(ba.userData,"Abort")==0)
//							print "Aborting Loop ", tn, mm
							DoUpdate/W=$cpanel/E=0
							ba.eventCode=2
							DoLoop(ba)
							break
						else
							if (fw)
								tn = 0
							else
								tn = inf
							endif
						endif
					while(1)
				else
					LoopThruTraces(ba)
//					print "Ending ", tn, mm
					DoUpdate/W=$cpanel/E=0
					ba.eventCode=2
					DoLoop(ba)
				endif
			endif
			break
	endswitch
	
	return 0
End

// loop thru traces as a movie

Function LoopThruTraces(ba)
	STRUCT WMButtonAction &ba
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	SVAR pn = cwDFR:panelname
	NVAR tn = cwDFR:tracenumber
	NVAR sp = cwDFR:speed

	STRUCT WMSetVariableAction sva
	
	variable nt, ic, ks, st, step=1
	string cpanel = cw + "#" + pn
	
	nt = ItemsInList(scrtTraceNameList())
	st = tn>=nt ? nt-1 : tn
	sva.eventCode=99
	DoUpdate/W=$(cw + "#" + pn)/E=1
	if (cmpStr(ba.ctrlName,"LoopForward")==0)
		try
			for (ic=st;ic<nt;ic+=1)
			
				// do checks for abort

				if (GetKeyState(0) & 32)
					ba.eventCode=2
					ba.userData = "Abort"
//					print "Escaping ", tn, mm
					break
				endif
				
				// go to trace

				tn = ic
				GoToTraceNumber(sva)
				
				// update panel and check for button press
				
				DoUpdate/W=$(cw + "#" + pn)
//				print "V_flag ", V_flag
				if (V_flag==2)
					ba.eventCode=2
					ba.userData = "Abort"
//					print "button press"
					break
				endif
				
				// sleep
				
				Sleep/S 2/(sp)
			endfor
		catch
			// abort via some other reason
			ba.eventCode=2
			ba.userData = "Abort"
			return 0
		endtry
	else
		try
			for (ic=st;ic>=0;ic+=-1)
			
				// do checks for abort

				if (GetKeyState(0) & 32)
					ba.eventCode=2
					ba.userData = "Abort"
//					print "Escaping ", tn, mm
					break
				endif
				
				// go to trace

				tn = ic
				GoToTraceNumber(sva)
				
				// update panel and check for button press
				
				DoUpdate/W=$cpanel
//				print "V_flag ", V_flag
				if (V_flag==2)
					ba.eventCode=2
					ba.userData = "Abort"
//					print "button press"
					break
				endif
				
				// sleep
				
				Sleep/S 2/(sp)
			endfor
		catch
			// abort via some other reason
			ba.eventCode=2
			ba.userData = "Abort"
			return 0
		endtry
	endif
	return 0
end

// clean up graph and kill things when panel closed

Function CleanUpandKill()
	
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	SVAR pn = cwDFR:panelname

	SetWindowHooks(-1)
	LoadScrollTracesPlugIns(0)
	KillWindow $(cw + "#" + pn)
	ModifyGraph/W=$cw hideTrace=0
	RemoveFromGraph/Z/W=$cw scrt_frozen
	SetAxis/W=$cw/A/Z
	KillDataFolder root:Packages:ScrollTraces:$(cw)
	return 0
end

// general hook function for graph or panel

Function ScrollTracesHook(sh)
	STRUCT WMWinHookStruct &sh
	
	NVAR/SDFR=$thePackageFolder mm = moviemode

	// global movie mode not handled

	variable hr = 0
	
	if (mm)
		return hr
	endif

	SVAR/SDFR=$thePackageFolder cw = currWin
	
	// activating the graph or panel sets its graph current window
	// kill requests (with active panels) first kill panel
	
//	GetWindow $sh.winName activeSW
//	print "check Hook ", S_Value, sh.eventCode
	
	switch (sh.eventCode)
		case 0:	// activate the panel
			cw = WinName(0,1,1)
			hr = 1
			break
		case 17:
			CleanUpandKill()
			break
	endswitch
	
	return hr
end

// hook function that handles keyboard events in non-movie mode

Function ScrollTracesKeyboardHook(sh)
	STRUCT WMWinHookStruct &sh
	
	NVAR/SDFR=$thePackageFolder mm = moviemode
	SVAR/SDFR=$thePackageFolder cw = currWin

	DFREF cwDFR = $(thePackageFolder + ":" + cw)

	NVAR tn = cwDFR:tracenumber
	NVAR sp = cwDFR:speed
	NVAR kb = cwDFR:keyboard
	SVAR pn = cwDFR:panelname

	STRUCT WMSetVariableAction sva
	STRUCT WMCheckboxAction cba
	
	variable hr = 0
	
	// global movie mode not handled

	if (mm)
		return hr
	endif
	
	// handle only k to reactivate keyboard input
	if (!kb)
		if (sh.keycode==107)
			kb = 1
			Checkbox keyboard,  win=$(cw + "#" + pn), value = 1
			hr = 1
		endif
		return hr
	endif
	
	// keyboard requests of an active graph or panel (non-movie mode)
	
//	GetWindow $sh.winName activeSW
//	print "check Keyboard Hook ", S_Value
	
	if (sh.eventCode == 11)
		//print sh.keycode
		switch(sh.keycode)
			case 31:	// up key
				sp += -1
				if (sp<1)
					sp = 1
				endif
				hr = 1
				break
			case 30:	// down key
				sp += 1
				if (sp>20)
					sp = 20
				endif
				hr = 1
				break
			case 29:	// right key
				sva.eventCode=99
				if (GetKeyState(0) & 4)
					tn = inf
				else
					tn += 1
				endif
				GoToTraceNumber(sva)
				hr = 1
				break
			case 28:	// left key
				sva.eventCode=99
				if (GetKeyState(0) & 4)
					tn = 0
				else
					tn += -1
				endif
				GoToTraceNumber(sva)
				hr = 1
				break
			case 97:	// a key
				cba.ctrlName = "scale_a"
				cba.eventCode = 99
				SetScaleMode(cba)
				hr = 1
				break
			case 99:	// c key
				cba.ctrlName = "scale_c"
				cba.eventCode = 99
				SetScaleMode(cba)
				hr = 1
				break
			case 102: // f key
				cba.ctrlName = "scale_f"
				cba.eventCode = 99
				SetScaleMode(cba)
				hr = 1
				break
			case 104: // h key
				cba.ctrlName = "scale_h"
				cba.eventCode = 99
				SetScaleMode(cba)
				hr = 1
				break
			case 107: // k key
				kb = 0
				Checkbox keyboard, win=$(cw + "#" + pn), value=0
				hr = 1
				break
			case 114: // r key
				cba.ctrlName = "scale_r"
				cba.eventCode = 99
				SetScaleMode(cba)
				hr = 1
				break
			case 111: // o key
				cba.ctrlName = "once"
				cba.eventCode = 99
				SetLoopMode(cba)
				hr = 1
				break
			case 108: // l key
				cba.ctrlName = "loop"
				cba.eventCode = 99
				SetLoopMode(cba)
				hr = 1
				break
		endswitch
	endif
	
	return hr
end

// hook function that handles movie mode

Function ScrollTracesMovieModeHook(sh)
	STRUCT WMWinHookStruct &sh
	
	variable hr = 0

	// only movie mode handled here (for both graph and panel)
	// prevents killing of the active graph and panel
	
//	print "watching movie ", sh.eventCode
	if (sh.eventCode == 17)
		hr = 2
	endif
	
	return hr
end

Function/S get_NPlugInsMenu()

	variable npd, npi
	string npimstr
	
	npd = ItemsInList(FunctionList("ScrTracesPI_Default",";","WIN:[ProcGlobal]"))
	npi = npd + 10*ItemsInList(FunctionList("ScrTracesPI_*",";","WIN:[ProcGlobal]"))

	switch(npi)
		case 0:
			npimstr = "-;"
			break
		case 1:
			npimstr = "-;+;"
			break
		default:
			if (mod(npi,10)==0)
				npimstr = "-;!;"
			else
				npimstr = "-;+;!;"			
			endif
			break
	endswitch

	return npimstr
end